@@ -12,12 +12,11 @@
 #include <gtk/gtk.h>
 #include <linux/videodev.h>
 #include <png.h>
+
 #include "gqcam.h"
 #include "frontend.h"
 #include "color.h"
 
-
-
 void create_controls_color(struct Camera *camera)
 {
 /*
@@ -36,9 +35,7 @@ void create_controls_color(struct Camera *camera)
   GtkWidget *scalehue;
   GtkObject *adjcolor;
   GtkWidget *scalecolor;
-  GtkWidget *radiobutton;
   GSList *camera->controls.radiogroup;
-  GtkWidget *button;
 */
   int ib=130, ic=30, iw=150, i, count=32;
   gint context_id;
@@ -128,12 +125,16 @@ void create_controls_color(struct Camera *camera)
   gtk_container_add (GTK_CONTAINER (camera->controls.frame), camera->controls.scalecolor);
   gtk_widget_show (camera->controls.scalecolor);
 
-  /* Create sizedepthbox */
+  /*********************************************************/
+  /* Size box (with three radio buttons Full/Half/Quarter) */
+  /*********************************************************/
+  GtkWidget *tmp1, *tmp2, *tmp3; /* temp pointer to build widget */
+
   camera->controls.sizedepthbox = gtk_vbox_new (FALSE, 0);
   gtk_box_pack_start( GTK_BOX (camera->controltop), camera->controls.sizedepthbox, 1, 1, 0);
   gtk_widget_show (camera->controls.sizedepthbox);
 
-  /* This is the size... */
+  /* Size */
   camera->controls.frame = gtk_frame_new("Size");
   gtk_box_pack_start( GTK_BOX (camera->controls.sizedepthbox), camera->controls.frame, 1, 1, 0);
   gtk_widget_show (camera->controls.frame);
@@ -142,38 +143,31 @@ void create_controls_color(struct Camera *camera)
   gtk_container_add (GTK_CONTAINER (camera->controls.frame), camera->controls.innerbox);
   gtk_widget_show (camera->controls.innerbox);
 
-  camera->controls.radiobutton = gtk_radio_button_new_with_label (NULL, "Full");
-  gtk_box_pack_start (GTK_BOX (camera->controls.innerbox), camera->controls.radiobutton, TRUE, TRUE, 0);
-
+  /* Full */
+  tmp1 = gtk_radio_button_new_with_label (NULL, "Full");
+  gtk_box_pack_start(GTK_BOX(camera->controls.innerbox), tmp1, TRUE, TRUE, 0);
   if (camera->vid_win.width == camera->vid_caps.maxwidth)
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (camera->controls.radiobutton), TRUE);
-  gtk_signal_connect(GTK_OBJECT (camera->controls.radiobutton), "pressed", GTK_SIGNAL_FUNC(setsizefull), camera);
-
-  gtk_widget_show (camera->controls.radiobutton);
-  
-  camera->controls.radiogroup = gtk_radio_button_group (GTK_RADIO_BUTTON (camera->controls.radiobutton));
-  camera->controls.radiobutton = gtk_radio_button_new_with_label(camera->controls.radiogroup, "Half");
-  gtk_box_pack_start (GTK_BOX (camera->controls.innerbox), camera->controls.radiobutton, TRUE, TRUE, 0);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp1), TRUE);
+  gtk_signal_connect(GTK_OBJECT(tmp1), "pressed", GTK_SIGNAL_FUNC(setsizefull), camera);
+  gtk_widget_show(tmp1);
 
+  /* Half */
+  tmp2 = gtk_radio_button_new_with_label(
+	   gtk_radio_button_group(GTK_RADIO_BUTTON(tmp1)), "Half");
+  gtk_box_pack_start (GTK_BOX(camera->controls.innerbox), tmp2, TRUE, TRUE, 0);
   if (camera->vid_win.width == (camera->vid_caps.maxwidth)/2)
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (camera->controls.radiobutton), TRUE);
-
-  gtk_signal_connect (GTK_OBJECT (camera->controls.radiobutton), "pressed",
-		      (GtkSignalFunc) setsizehalf, camera);
-
-  //  gtk_signal_connect(GTK_OBJECT (camera->controls.radiobutton), "clicked", (GtkSignalFunc)setsizehalf, camera);
-
-  gtk_widget_show (camera->controls.radiobutton);
-
-  camera->controls.radiobutton = gtk_radio_button_new_with_label( gtk_radio_button_group (GTK_RADIO_BUTTON (camera->controls.radiobutton)), "Quarter");
-  gtk_box_pack_start (GTK_BOX (camera->controls.innerbox), camera->controls.radiobutton, TRUE, TRUE, 0);
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tmp2), TRUE);
+  gtk_signal_connect(GTK_OBJECT(tmp2), "pressed", (GtkSignalFunc)setsizehalf, camera);
+  gtk_widget_show(tmp2);
 
+  /* Quarter */
+  tmp3 = gtk_radio_button_new_with_label(
+         gtk_radio_button_group(GTK_RADIO_BUTTON(tmp1)), "Quarter");
+  gtk_box_pack_start(GTK_BOX(camera->controls.innerbox), tmp3, TRUE, TRUE, 0);
   if (camera->vid_win.width == (camera->vid_caps.maxwidth)/4)
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (camera->controls.radiobutton), TRUE);
-  gtk_signal_connect(GTK_OBJECT (camera->controls.radiobutton), "pressed", (GtkSignalFunc)setsizequarter, camera);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp3), TRUE);
+  gtk_signal_connect(GTK_OBJECT(tmp3), "pressed", (GtkSignalFunc)setsizequarter, camera);
+  gtk_widget_show(tmp3);
 
-  gtk_widget_show (camera->controls.radiobutton);
-
-  gtk_widget_show (camera->controltop);
-  
+  gtk_widget_show (camera->controltop);  
 }
