@@ -1,3 +1,21 @@
+/*
+ *  frontend.c 
+ *
+ *  Copyright (c) 2005 Giansalvo Gusinu <giansalvo at gusinu.net>
+ *  Copyright (C) 1999 Cory Lueninghoener (cluenin1@bigred.unl.edu)
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ */
+
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -14,6 +32,7 @@
 #include <glib.h>
 #include <linux/videodev.h>
 #include <png.h>
+
 #include "gqcam.h"
 #include "frontend.h"
 #include "preferences.h"
@@ -23,6 +42,15 @@
 #include "timer.h"
 
 
+/* list of v4l palette recognized by gqcam */
+static struct symbolic_list v4l_palette_plist[] = {
+	{ VIDEO_PALETTE_GREY,   "GREY" },
+	{ VIDEO_PALETTE_RGB24,  "RGB24" },
+	{ VIDEO_PALETTE_YUV420, "YUV420" },
+       	{ VIDEO_PALETTE_YUV420P,"YUV420P" },
+	{ -1,NULL }
+};
+
 void closedialog(GtkWidget *widget, GtkWidget *dialog)
 {
   gtk_widget_destroy(dialog);
@@ -302,15 +330,12 @@ void camera_info(GtkWidget *widget, struct Camera *camera)
   gtk_text_insert(GTK_TEXT(textbox), NULL, NULL, NULL, buff, -1);
   sprintf(buff, "Depth:\t\t%i\n", camera->vid_pic.depth);
   gtk_text_insert(GTK_TEXT(textbox), NULL, NULL, NULL, buff, -1);
-  sprintf(buff, "Palette:\t%i\n", camera->vid_pic.palette);
+  sprintf(buff, "Palette:\t%i - %s\n",camera->vid_pic.palette, symbolic(v4l_palette_plist, camera->vid_pic.palette));
+
   gtk_text_insert(GTK_TEXT(textbox), NULL, NULL, NULL, buff, -1);
 
   gtk_adjustment_set_value(GTK_ADJUSTMENT(GTK_TEXT(textbox)->vadj), 0);
 
-
-
-  //  print_cam_info(camera);
-
 }
 
 
@@ -320,10 +345,10 @@ void about_dialog(GtkWidget *widget, struct Camera *camera)
   char buff[80];
 
   dialog = gtk_dialog_new();
-  sprintf(buff, "Gqcam version %s\n(C)2000 Cory Lueninghoener", VERSION);
+  sprintf(buff, "gqcam %s\n\n(C)2000 Cory Lueninghoener\n(C)2005 Giansalvo Gusinu", VERSION);
   
   label = gtk_label_new (buff);
-  button = gtk_button_new_with_label("Thanks, Cory!");
+  button = gtk_button_new_with_label("OK");
      
   gtk_window_set_title(GTK_WINDOW (dialog),"Gqcam - About");
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
@@ -545,7 +570,7 @@ int create_frontend(struct Camera *camera)
 
   guchar *gbuff;
   GdkColor c;
-  char title[50] = "Gqcam - ";
+  char title[50] = "gqcam - ";
   struct video_clip vid_clips[32];
   int ib=130, ic=30, iw=150, i, count=32;
 
@@ -778,6 +803,3 @@ int create_frontend(struct Camera *camera)
 
   return 0;
 }
-
-
-
