@@ -19,27 +19,3 @@ void clearimg(struct Camera*);
 void update_controls(struct Camera*);
 void dock_controls(GtkWidget*, struct Camera*);
 void closedialog(GtkWidget *widget, GtkWidget *dialog);
-
-/*
-struct Controls
-{
-  GtkWidget *outerbox;
-  GtkWidget *innerbox;
-  GtkWidget *scalebox;
-  GtkWidget *sizedepthbox;
-  GtkWidget *frame;
-  GtkObject *adjbrightness;
-  GtkWidget *scalebrightness;
-  GtkObject *adjwhite;
-  GtkWidget *scalewhite;
-  GtkObject *adjcontrast;
-  GtkWidget *scalecontrast;
-  GtkObject *adjhue;
-  GtkWidget *scalehue;
-  GtkObject *adjcolor;
-  GtkWidget *scalecolor;
-  GtkWidget *radiobutton;
-  GSList *radiogroup;
-  GtkWidget *button;
-};
-*/
