@@ -1,20 +1,19 @@
-/*  gqcam.c - The frontend to Gqcam, a GTK based QuickPict clone.
-    Copyright (C) 1999 Cory Lueninghoener (cluenin1@bigred.unl.edu)
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
+/*
+ *  gqcam.c 
+ *
+ *  Copyright (c) 2005 Giansalvo Gusinu <giansalvo at gusinu.net>
+ *  Copyright (C) 1999 Cory Lueninghoener (cluenin1@bigred.unl.edu)
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
  */
 
 #include <unistd.h>
@@ -34,14 +33,16 @@
 #include <linux/videodev.h>
 #include <signal.h>
 #include <png.h>
+
 #include "gqcam.h"
 #include "frontend.h"
 #include "save.h"
 #include "filters.h"
+#include "jpeg.h"
+#include "yuv_to_rgb.h"
 
 char version[] = VERSION;
 
-
 void init_cam(struct Camera *camera)
 {
   camera->greyscale = 0;
@@ -52,7 +53,8 @@ void init_cam(struct Camera *camera)
   camera->frozen = 0;
   camera->update_camera = 0;
   camera->saving = 0;
-  camera->savetype = PNG;
+  camera->inputtype = INPUT_NOT_FORCED;
+  camera->savetype = SAVE_PNG;
   camera->capture = 1;
   camera->dev = 0;
   strcpy(camera->devname, "/dev/video");
@@ -117,7 +119,8 @@ void get_cam_info(struct Camera *camera)
   }
 }
 
-
+/* FIXME this can be used for debugging. see caminfo() in frontend.c */
+/*
 void print_cam_info(struct Camera *camera)
 {
   printf("Name: %s\n", camera->vid_caps.name);
@@ -152,39 +155,30 @@ void print_cam_info(struct Camera *camera)
   if (camera->vid_caps.type & VID_TYPE_SUBCAPTURE) {
     printf("\tCan capture subareas of the image\n");
   }
-  
   printf("Channels: %i\n", camera->vid_caps.channels);
   printf("Audios: %i\n", camera->vid_caps.audios);
   printf("Maxwidth: %i\n", camera->vid_caps.maxwidth);
   printf("Maxheight: %i\n", camera->vid_caps.maxheight);
   printf("Minwidth: %i\n", camera->vid_caps.minwidth);
   printf("Minheight: %i\n", camera->vid_caps.minheight);
-
   printf("---------\n");
-  
   printf("X: %i\n", camera->vid_win.x);
   printf("Y: %i\n", camera->vid_win.y);
   printf("Width: %i\n", camera->vid_win.width);
   printf("Height: %i\n", camera->vid_win.height);
   printf("Chromakey: %i\n", camera->vid_win.chromakey);
   printf("Flags: %i\n", camera->vid_win.flags);
-/*
-  printf("Clips: %i\n", camera.vid_win.clips);
-  printf("Clipcound: %i\n", camera.vid_win.clipcount);
-*/
-
   printf("---------\n");
-  
   printf("Brightness:\t%i (%i)\n", camera->vid_pic.brightness, camera->vid_pic.brightness/256);
   printf("Hue:\t\t%i (%i)\n", camera->vid_pic.hue, camera->vid_pic.hue/256);
   printf("Color:\t\t%i (%i)\n", camera->vid_pic.colour, camera->vid_pic.colour/256);
   printf("Contrast:\t%i (%i)\n", camera->vid_pic.contrast, camera->vid_pic.contrast/256);
   printf("Whiteness:\t%i (%i)\n", camera->vid_pic.whiteness, camera->vid_pic.whiteness/256);
   printf("Depth:\t\t%i\n", camera->vid_pic.depth);
-  printf("Palette:\t%i\n", camera->vid_pic.palette);
-
+  printf("Palette:\t%i\n",camera->vid_pic.palette);
   return;
 }
+*/
 
 void open_cam(struct Camera *camera) 
 {
@@ -192,7 +186,7 @@ void open_cam(struct Camera *camera)
     camera->dev = open(camera->devname, O_RDWR);
     //    printf("Opening: %d\n", camera->dev);
     if (camera->dev < 0) {
-      perror("/dev/video");
+      perror(camera->devname);
       exit(1);
     }
   }
@@ -216,6 +210,7 @@ void display(struct Camera *camera)
   int xpos=0, ypos=0;
   GdkRectangle update_rec;
   char tbuff[1024];
+  static int frameno = 0;
 
   while( !plsquit ) {
     if( !sem_wait( &s_draw ) ) {
@@ -251,7 +246,8 @@ void display(struct Camera *camera)
 
       gdk_threads_enter();
       gtk_statusbar_pop( GTK_STATUSBAR( camera->statusbar ), 12 );
-      sprintf( tbuff, "Speed (fps) - Average: %d  Current: %d", camera->fps_avg, camera->fps_current );
+      sprintf( tbuff, "Speed (fps) - Average: %d  Current: %d. Frame no %d", 
+	       camera->fps_avg, camera->fps_current, frameno++);
       gtk_statusbar_push( GTK_STATUSBAR( camera->statusbar ), 12, tbuff );
       gtk_widget_draw (camera->drawing_area, &update_rec);
       gdk_threads_leave();
@@ -262,12 +258,14 @@ void display(struct Camera *camera)
   return;
 }
 
-
 void grab_image(struct Camera *camera)
 {
-  int len;
+  const JPEG_HEADER_SIZE = 2;
+
   GdkRectangle update_rec;
   GdkEventExpose *event;
+  int input_type;
+  struct ov511_frame temp;
 
   get_cam_info(camera);
 
@@ -278,6 +276,7 @@ void grab_image(struct Camera *camera)
 
   while ( !plsquit ) {
     // order matters! the sem_waits MUST be before the mutex lock
+    
     if( !sem_wait( &s_grab1 ) && ( camera->speed_fastest || !sem_wait( &s_grab2 ) ) && !pthread_mutex_lock( &camera->freeze_mutex ) && !pthread_mutex_lock( &camera->iscam_mutex )){
 
       pthread_mutex_lock( &camera->pref_mutex );
@@ -285,34 +284,76 @@ void grab_image(struct Camera *camera)
 	set_cam_info(camera);
 	get_cam_info(camera);
 
-	
 	camera->update_camera = 0;
       }
       pthread_mutex_unlock( &camera->pref_mutex );
-      
-      if( camera->dev )
-	len = read (camera->dev, camera->picbuff, camera->vid_caps.maxwidth * camera->vid_caps.maxheight*3);
-      if (len <= 0)
+
+      if( camera->dev ){
+
+	/* 
+	 * select actual input type based on user option, device capabilities and default.
+	 * FIXME this info should rather be kept updated (when changing camera or settings)
+	 * and not calculated here!
+	 */
+	if (camera->inputtype != INPUT_NOT_FORCED)
+	  input_type = camera->inputtype;
+	else
+	  switch (camera->vid_pic.palette) {
+	  case VIDEO_PALETTE_RGB24:  /* TESTED, it works! */
+	  case VIDEO_PALETTE_RGB565: /* not tested, will it work like RGB24? */
+	  case VIDEO_PALETTE_RGB555: /* not tested, will it work like RGB24? */
+	  case VIDEO_PALETTE_RGB32:  /* not tested, will it work like RGB24? */
+	    input_type = INPUT_RGB;
+	    break;
+	  case VIDEO_PALETTE_YUV420: /* TESTED, it works! and it is used as default */
+	  default:
+	    input_type = INPUT_YUV;
+	    break;
+	  }
+
+	  switch(input_type) {
+	  case INPUT_YUV:
+            camera->img_size = read (camera->dev, camera->rawbuffer, camera->vid_caps.maxwidth * camera->vid_caps.maxheight * 3);
+	    temp.width  = temp.rawwidth  = camera->vid_caps.maxwidth;
+	    temp.height = temp.rawheight = camera->vid_caps.maxheight;
+	    yuv420p_to_rgb(&temp, camera->rawbuffer, camera->picbuff, 24);
+	    break;
+	  case INPUT_RGB:
+            camera->img_size = read (camera->dev, camera->picbuff, camera->vid_caps.maxwidth * camera->vid_caps.maxheight * 3);
+	    break;
+	  case INPUT_JPEG:
+	    camera->img_size = read (camera->dev, camera->rawbuffer, sizeof(camera->rawbuffer)) - JPEG_HEADER_SIZE; // TODO error management
+	    camera->img_size = decompress_JPEG(camera->rawbuffer + JPEG_HEADER_SIZE, camera->img_size, camera->picbuff); // TODO error management
+	    break;
+	  default:
+	    //TODO internal error
+	    fprintf(stderr, "grab_image: incorrect input_type %d\n", input_type);
+	    break;
+	  }
+      }
+      if (camera->img_size <= 0)
 	fprintf(stderr, "Error reading image...\n");      
     }
     pthread_mutex_unlock( &camera->freeze_mutex );
     pthread_mutex_unlock( &camera->iscam_mutex );
-    if( camera->dump )
+    if( camera->dump ) {
+      //fprintf(stderr, "gqcam.c: camera->dump\n");
       return;
+    }
     if(camera->on_timer){
       savefile_append_time(camera);
       if(camera->timer_struct.beep)
 	gdk_beep();
       switch (camera->savetype) {
-      case PPM:
+      case SAVE_PPM:
 	ppm_save(camera);
 	break; 
     
-      case JPEG:
+      case SAVE_JPEG:
 	jpeg_save(camera);	
 	break;
 	
-      case PNG:
+      case SAVE_PNG:
 	png_save(camera);
 	break;
       }
@@ -322,7 +363,7 @@ void grab_image(struct Camera *camera)
       }
     }
     sem_post( &s_draw );
-  }
+  } 
 }
   
 void delete_event(GtkWidget *widget, struct Camera *camera)
@@ -387,7 +428,6 @@ int main(int argc, char *argv[])
     read_pref_file(preffile, &camera);
     fclose(preffile);
   }
-  
 
   while( !done ) {
     static struct option long_options[] = 
@@ -404,11 +444,12 @@ int main(int argc, char *argv[])
       { "contrast", required_argument, NULL, 'c' },
       { "maxfps", required_argument, NULL, 'm' },
       { "fullspeed", no_argument, NULL, 'F' },
+      { "input", required_argument, NULL, 'i' },
       { 0, 0, 0, 0 }
     };
     int c;
 
-    c = getopt_long( argc, argv, "FhVsav:d:b:w:c:m:t:", long_options, NULL );
+    c = getopt_long( argc, argv, "FhVsav:d:b:w:c:m:t:i:", long_options, NULL );
     switch ( c ) {
     case 'h' :
       print_usage();
@@ -434,11 +475,13 @@ int main(int argc, char *argv[])
       break;
     case 't' :
       if(!strcmp(optarg, "PPM"))
-	camera.savetype = PPM;
+	camera.savetype = SAVE_PPM;
       else if(!strcmp(optarg, "PNG"))
-	camera.savetype = PNG;
+	camera.savetype = SAVE_PNG;
       else if(!strcmp(optarg, "JPEG"))
-	camera.savetype = JPEG;
+	camera.savetype = SAVE_JPEG;
+      else if(!strcmp(optarg, "RAW"))
+	camera.savetype = SAVE_RAW;
       break;
     case 'b' :
       brightness = atoi( optarg );
@@ -457,6 +500,14 @@ int main(int argc, char *argv[])
       camera.speed_fastest = 1;
       camera.timeout = 0;
       break;
+    case 'i' :
+      if(!strcmp(optarg, "JPEG"))
+	camera.inputtype = INPUT_JPEG;
+      else if (!strcmp(optarg, "RGB"))
+	camera.inputtype = INPUT_RGB;
+      else if (!strcmp(optarg, "YUV"))
+	camera.inputtype = INPUT_YUV;
+      break;
     case '?' :
       fprintf( stderr, "invalid option, or ambiguous argument\n" );
       print_usage();
@@ -520,25 +571,23 @@ int main(int argc, char *argv[])
 }
 
 void print_usage() {
-      fprintf(stderr, "gqcam - GTK QuickCam Control\n");
-      fprintf(stderr, "Copyright (C) 1999  Cory Lueninghoener <cluenin1@bigred.unl.edu>\n");
-      fprintf(stderr, "This software comes with ABSOLUTELY NO WARRANTY\n");
-      fprintf(stderr, "This software is free software, and you are welcome to redistribute it\n");
-      fprintf(stderr, "under certain conditions\n");
-      fprintf(stderr, "See the README file for a more complete notice.\n");
-      fprintf( stderr, "usage: gqcam [-hVFs] [-d file] [-c contrast] [-w whiteness] [-b brightness] [-m maxfps] [-d filename] [-t type]\n" );
+      fprintf(stderr, "gqcam %s - GTK QuickCam Control\n", VERSION);
+      fprintf(stderr, "Copyright (C) 2005  Giansalvo Gusinu <giansalvo at gusinu.net>\n");
+      fprintf(stderr, "Copyright (C) 1999  Cory Lueninghoener <cluenin1@bigred.unl.edu>\n\n");
+      fprintf(stderr, "usage: gqcam [-hVFs] [-d file] [-c contrast] [-w whiteness] [-b brightness] [-m maxfps] [-d filename] [-t type] [-i format]\n\n");
       fprintf(stderr, "\t-h --help\t\tdisplay this help screen\n");
       fprintf(stderr, "\t-V --version\t\tdisplay version and exit\n");
-      fprintf(stderr, "\t-s --swap\t\tswap RGB into BGR (WARNING: this option will disappear in the future, turning into a filter instead)\n");
+      fprintf(stderr, "\t-s --swap\t\tswap RGB into BGR\n");
       fprintf(stderr, "\t-a --autobright\tperform autobrightness\n");
-      fprintf(stderr, "\t-v --video <video dev>\tgrab frames from <video dev>\n");
+      fprintf(stderr, "\t-v --video <video dev>\tgrab frames from <video dev> [/dev/video]\n");//FIXME
       fprintf(stderr, "\t-d --dump <filename>\tdump image to <filename> \n\t\t\t\t(or stdout if filename is \"-\")\n");
-      fprintf(stderr, "\t-t --type <type>\ttype of image to dump(default:PNG)\n\t\t\t\t(valid types: PPM PNG JPEG)\n");
+      fprintf(stderr, "\t-t --type <type>\ttype of image to dump: [PNG], PPM,JPEG, RAW)\n");
       fprintf(stderr, "\t-b --brightness <num>\tset brightness to <num>\n");
       fprintf(stderr, "\t-w --whiteness <num>\tset white balance to <num>\n");
       fprintf(stderr, "\t-c --contrast <num>\tset contrast to <num>\n");
-      fprintf(stderr, "\t-m --maxfps <num>\tset the maximum frames per second(default:%d)\n", 10 );
+      fprintf(stderr, "\t-m --maxfps <num>\tset the maximum frames per second [%d]\n", 10 );//FIXME
       fprintf(stderr, "\t-F --fullspeed\t\tdisplay as many frames per second as you can\n" );
+      fprintf(stderr, "\t-i --input\t\tforce input format: YUV, RGB, JPEG\n" );
 }
 
 int dump_pict( struct Camera *camera, char *filename, int brightness, int contrast, int whiteness )
@@ -573,21 +622,24 @@ int dump_pict( struct Camera *camera, char *filename, int brightness, int contra
   close_cam(camera, 1);
   
   gdk_threads_enter();
-  if(camera->savetype == JPEG){
+  if(camera->savetype == SAVE_JPEG){
     camera->save_struct.smoothness = 0;
     camera->save_struct.quality = 75;
     camera->save_struct.optimize = 0;
     jpeg_save(camera);
   }
-  else if(camera->savetype == PNG){
+  else if(camera->savetype == SAVE_PNG){
     camera->save_struct.interlace = PNG_INTERLACE_NONE;
     camera->save_struct.compression = 6;
     png_save(camera);
   }
-  else if(camera->savetype == PPM){
-    camera->save_struct.format = RAW;
+  else if(camera->savetype == SAVE_PPM){
+    camera->save_struct.format = PPM_RAW;
     ppm_save(camera);
   }
+  else if(camera->savetype == SAVE_RAW) {
+	  raw_save(camera);
+  }
 
   gdk_threads_leave();
 
