@@ -1,3 +1,28 @@
+/*
+ *  gqcam.h
+ *
+ *  Copyright (c) 2005 Giansalvo Gusinu <giansalvo at gusinu.net>
+ *  Copyright (C) 1999 Cory Lueninghoener (cluenin1@bigred.unl.edu)
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ */
+
+enum Input_Types {
+  INPUT_NOT_FORCED = 0,
+  INPUT_JPEG       = 1,
+  INPUT_RGB        = 2,
+  INPUT_YUV        = 3
+};
+
 struct Pref_Dialog
 {
   GtkWidget *prefdialog;
@@ -10,6 +35,9 @@ struct Pref_Dialog
   GtkWidget *filterspage;
   GtkWidget *filterstab;
   GtkWidget *rgbswap;
+  GtkWidget *force_YUV;
+  GtkWidget *force_RGB;
+  GtkWidget *force_JPEG;
 
   GtkWidget *button;
   GtkWidget *label;
@@ -125,12 +153,8 @@ struct Controls
   GtkWidget *scalehue;
   GtkObject *adjcolor;
   GtkWidget *scalecolor;
-  GtkWidget *radiobutton;
-  GSList *radiogroup;
-  GtkWidget *button;
 };
 
-
 struct Camera
 {
   int dev;
@@ -140,6 +164,7 @@ struct Camera
   int update_camera;
   int capture;
   int saving;
+  int inputtype;
   int savetype;
   int docked;
   int dump;
@@ -174,10 +199,41 @@ struct Camera
   struct Timer_Struct timer_struct;
   struct Controls controls;
   struct Pref_Dialog pref_dialog;
+  int img_size;
+
+  /* 
+   * buffer for raw data before processing (YUV and jpeg)
+   * FIXME use dynamically allocated memory, use more appropriate size,
+   * FIXME probably this could be local to grab_image()
+   */
+  unsigned char rawbuffer[640*480*3]; 
 };
 
 
 
+/* Used to represent a list of values and their respective symbolic names */
+struct symbolic_list {
+	int num;
+	char *name;
+};
+
+#define NOT_DEFINED_STR "Unknown"
+
+/* Returns the name of the matching element in the symbolic_list array. The
+ * end of the list must be marked with an element that has a NULL name.
+ */
+static inline char * 
+symbolic(struct symbolic_list list[], int num)
+{
+	int i;
+
+	for (i = 0; list[i].name != NULL; i++)
+			if (list[i].num == num)
+				return (list[i].name);
+
+	return (NOT_DEFINED_STR);
+}
+
 void display(struct Camera*);
 void delete_event(GtkWidget*, struct Camera *camera);
 void open_cam(struct Camera*);
