@@ -1,3 +1,21 @@
+/*
+ *  greyscale.c 
+ *
+ *  Copyright (c) 2005 Giansalvo Gusinu <giansalvo at gusinu.net>
+ *  Copyright (C) 1999 Cory Lueninghoener (cluenin1@bigred.unl.edu)
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ */
+
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
@@ -12,6 +30,7 @@
 #include <gtk/gtk.h>
 #include <linux/videodev.h>
 #include <png.h>
+
 #include "gqcam.h"
 #include "frontend.h"
 #include "greyscale.h"
@@ -20,6 +39,7 @@
 
 void create_controls_greyscale(struct Camera *camera) 
 {
+  GtkWidget *tmp0, *tmp; /* temporary pointer used to create the widgets */    
   /*
   GtkWidget *outerbox;
   GtkWidget *innerbox;
@@ -32,9 +52,6 @@ void create_controls_greyscale(struct Camera *camera)
   GtkWidget *scalewhite;
   GtkObject *adjcontrast;
   GtkWidget *scalecontrast;
-  GtkWidget *radiobutton;
-  GSList *radiogroup;
-  GtkWidget *button;
   */
 
   /* Create new outerbox */
@@ -95,7 +112,9 @@ void create_controls_greyscale(struct Camera *camera)
   gtk_box_pack_start( GTK_BOX (camera->controltop), camera->controls.sizedepthbox, 1, 1, 0);
   gtk_widget_show (camera->controls.sizedepthbox);
 
-  /* This is the size... */
+  /************************************************/
+  /* Size box with radio button Full/Half/Quarter */
+  /************************************************/
   camera->controls.frame = gtk_frame_new("Size");
   gtk_box_pack_start( GTK_BOX (camera->controls.sizedepthbox), camera->controls.frame, 1, 1, 0);
   gtk_widget_show (camera->controls.frame);
@@ -104,33 +123,31 @@ void create_controls_greyscale(struct Camera *camera)
   gtk_container_add (GTK_CONTAINER (camera->controls.frame), camera->controls.innerbox);
   gtk_widget_show (camera->controls.innerbox);
 
-  camera->controls.radiobutton = gtk_radio_button_new_with_label (NULL, "Full");
-  gtk_box_pack_start (GTK_BOX (camera->controls.innerbox), camera->controls.radiobutton, TRUE, TRUE, 0);
-
+  /* Full */
+  tmp0 = gtk_radio_button_new_with_label (NULL, "Full");
+  gtk_box_pack_start(GTK_BOX(camera->controls.innerbox), tmp0, TRUE, TRUE, 0);
   if (camera->vid_win.width == camera->vid_caps.maxwidth)
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (camera->controls.radiobutton), TRUE);
-  gtk_signal_connect(GTK_OBJECT (camera->controls.radiobutton), "pressed", GTK_SIGNAL_FUNC(setsizefull), camera);
-
-  gtk_widget_show (camera->controls.radiobutton);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp0), TRUE);
+  gtk_signal_connect(GTK_OBJECT(tmp0), "pressed", GTK_SIGNAL_FUNC(setsizefull), camera);
+  gtk_widget_show(tmp0);
   
-  camera->controls.radiogroup = gtk_radio_button_group (GTK_RADIO_BUTTON (camera->controls.radiobutton));
-  camera->controls.radiobutton = gtk_radio_button_new_with_label(camera->controls.radiogroup, "Half");
-  gtk_box_pack_start (GTK_BOX (camera->controls.innerbox), camera->controls.radiobutton, TRUE, TRUE, 0);
+  //  camera->controls.radiogroup = gtk_radio_button_group(GTK_RADIO_BUTTON(tmp));
 
+  /* Half */
+  tmp = gtk_radio_button_new_with_label(gtk_radio_button_group(GTK_RADIO_BUTTON(tmp0)), "Half");
+  gtk_box_pack_start(GTK_BOX(camera->controls.innerbox), tmp, TRUE, TRUE, 0);
   if (camera->vid_win.width == (camera->vid_caps.maxwidth)/2)
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (camera->controls.radiobutton), TRUE);
-  gtk_signal_connect(GTK_OBJECT (camera->controls.radiobutton), "pressed", GTK_SIGNAL_FUNC(setsizehalf), camera);
-  
-  gtk_widget_show (camera->controls.radiobutton);
-
-  camera->controls.radiobutton = gtk_radio_button_new_with_label( gtk_radio_button_group (GTK_RADIO_BUTTON (camera->controls.radiobutton)), "Quarter");
-  gtk_box_pack_start (GTK_BOX (camera->controls.innerbox), camera->controls.radiobutton, TRUE, TRUE, 0);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp), TRUE);
+  gtk_signal_connect(GTK_OBJECT(tmp), "pressed", GTK_SIGNAL_FUNC(setsizehalf), camera);
+  gtk_widget_show (tmp);
 
+  /* Quarter */
+  tmp = gtk_radio_button_new_with_label(gtk_radio_button_group(GTK_RADIO_BUTTON(tmp0)), "Quarter");
+  gtk_box_pack_start(GTK_BOX(camera->controls.innerbox), tmp, TRUE, TRUE, 0);
   if (camera->vid_win.width == (camera->vid_caps.maxwidth)/4)
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (camera->controls.radiobutton), TRUE);
-  gtk_signal_connect(GTK_OBJECT (camera->controls.radiobutton), "pressed", GTK_SIGNAL_FUNC(setsizequarter), camera);
-
-  gtk_widget_show (camera->controls.radiobutton);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp), TRUE);
+  gtk_signal_connect(GTK_OBJECT(tmp), "pressed", GTK_SIGNAL_FUNC(setsizequarter), camera);
+  gtk_widget_show(tmp);
 
   /* ...and this is the depth */
   camera->controls.frame = gtk_frame_new("Depth");
@@ -141,25 +158,27 @@ void create_controls_greyscale(struct Camera *camera)
   gtk_container_add (GTK_CONTAINER (camera->controls.frame), camera->controls.innerbox);
   gtk_widget_show (camera->controls.innerbox);
 
-  camera->controls.radiobutton = gtk_radio_button_new_with_label (NULL, "4 bpp");
-  gtk_box_pack_start (GTK_BOX (camera->controls.innerbox), camera->controls.radiobutton, TRUE, TRUE, 0);
+  /*WARNING! code modified, but NOT tested (replaced "camera->controls.radiobutton" with "tmp") */
 
+  /* 4 bpp */
+  tmp0 = gtk_radio_button_new_with_label (NULL, "4 bpp");
+  gtk_box_pack_start (GTK_BOX (camera->controls.innerbox), tmp0, TRUE, TRUE, 0);
   if (camera->vid_pic.depth == 4)
-      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (camera->controls.radiobutton), TRUE);
-  gtk_signal_connect(GTK_OBJECT (camera->controls.radiobutton), "pressed", GTK_SIGNAL_FUNC(setdepth4), camera);
-
-  gtk_widget_show (camera->controls.radiobutton);
-
-  camera->controls.radiogroup = gtk_radio_button_group (GTK_RADIO_BUTTON (camera->controls.radiobutton));
-  camera->controls.radiobutton = gtk_radio_button_new_with_label(camera->controls.radiogroup, "6 bpp");
-  gtk_box_pack_start (GTK_BOX (camera->controls.innerbox), camera->controls.radiobutton, TRUE, TRUE, 0);
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(tmp0), TRUE);
+  gtk_signal_connect(GTK_OBJECT(tmp0), "pressed", GTK_SIGNAL_FUNC(setdepth4), camera);
+  gtk_widget_show (tmp0);
 
+  /* create the radiogroup */
+  //  camera->controls.radiogroup = gtk_radio_button_group(GTK_RADIO_BUTTON(tmp0));
+  
+  /* 6 bpp */
+  tmp = gtk_radio_button_new_with_label(gtk_radio_button_group(GTK_RADIO_BUTTON(tmp0)), "6 bpp");
+  gtk_box_pack_start(GTK_BOX(camera->controls.innerbox), tmp, TRUE, TRUE, 0);
   if (camera->vid_pic.depth == 6)
-      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (camera->controls.radiobutton), TRUE);
-  gtk_signal_connect(GTK_OBJECT (camera->controls.radiobutton), "pressed", GTK_SIGNAL_FUNC(setdepth6), camera);
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp), TRUE);
+  gtk_signal_connect(GTK_OBJECT(tmp), "pressed", GTK_SIGNAL_FUNC(setdepth6), camera);
 
-  gtk_widget_show (camera->controls.radiobutton);
-
-  gtk_widget_show (camera->controltop);
+  gtk_widget_show(tmp);
 
+  gtk_widget_show(camera->controltop);
 }
