@@ -1,3 +1,21 @@
+/*
+ *  save.c 
+ *
+ *  Copyright (c) 2005 Giansalvo Gusinu <giansalvo at gusinu.net>
+ *  Copyright (C) 1999 Cory Lueninghoener (cluenin1@bigred.unl.edu)
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ */
+
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
@@ -14,11 +32,23 @@
 #include <stdio.h>
 #include <jpeglib.h>
 #include <png.h>
+
 #include "gqcam.h"
 #include "frontend.h"
 #include "save.h"
 #include "time.h"
 
+/* common file extensions added to filename */
+static struct symbolic_list fname_postfix_plist[] = {
+	{ SAVE_PNG, ".png" },
+	{ SAVE_PPM, ".ppm" },
+	{ SAVE_JPEG,".jpg" },
+	{ SAVE_GIF, ".gif" },
+	{ SAVE_RAW, ".raw" },
+	{ -1,NULL }
+};
+
+
 void select_ok(GtkWidget *widget, struct Camera *camera)
 {
   gtk_entry_set_text(GTK_ENTRY(camera->save_struct.textbox), gtk_file_selection_get_filename(GTK_FILE_SELECTION (camera->save_struct.fileselect)));
@@ -74,22 +104,25 @@ void save_ok(GtkWidget *widget, struct Camera *camera)
 
   if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(camera->save_struct.timebutton)))
     savefile_append_time(camera);
-  
+
+  strcat(camera->savefile,symbolic(fname_postfix_plist, camera->savetype)); // TODO add buffer overrun checking
+  fprintf(stderr, "save.c: saving to file %s\n", camera->savefile);
+
   switch (camera->savetype) {
 
-  case PPM:
+  case SAVE_PPM:
     if (GTK_TOGGLE_BUTTON(camera->page_ppm.radio_raw_ppm)->active){
-      camera->save_struct.format = RAW;
+      camera->save_struct.format = PPM_RAW;
     }
     else if (GTK_TOGGLE_BUTTON(camera->page_ppm.radio_ascii_ppm)->active){
-      camera->save_struct.format = ASCII;
+      camera->save_struct.format = PPM_ASCII;
     }
     if (camera->saving)
       ppm_save(camera);
 
     break; 
     
-  case JPEG:
+  case SAVE_JPEG:
     camera->save_struct.quality = GTK_ADJUSTMENT(camera->page_jpeg.quality_adj_jpeg)->value;
     camera->save_struct.smoothness = GTK_ADJUSTMENT(camera->page_jpeg.smooth_adj_jpeg)->value;
     camera->save_struct.optimize = (GTK_TOGGLE_BUTTON(camera->page_jpeg.optimize_jpeg)->active)?1:0;
@@ -97,13 +130,16 @@ void save_ok(GtkWidget *widget, struct Camera *camera)
       jpeg_save(camera);
 
     break;
-
-  case GIF:
     
+  case SAVE_RAW:
+ 	raw_save(camera);
+     break;
 
+  case SAVE_GIF:
+    // UNSUPPORTED
     break;
 
-  case PNG:
+  case SAVE_PNG:
     if(GTK_TOGGLE_BUTTON(camera->page_png.interlace_png)->active)
       camera->save_struct.interlace = PNG_INTERLACE_ADAM7;
     else
@@ -113,6 +149,10 @@ void save_ok(GtkWidget *widget, struct Camera *camera)
     if (camera->saving)
       png_save(camera);
     break;
+
+  default:
+    // TODO Report an internal error. Either I forgot to code this case or the type is incorrect.
+    break;
     
   }
 
@@ -346,7 +386,7 @@ void ppm_save(struct Camera *camera)
   }
  
   /*  Raw  */
-  if (camera->save_struct.format == RAW){
+  if (camera->save_struct.format == PPM_RAW){
     fprintf (outfile, "P6\n%d %d\n%d\n", camera->vid_win.width, camera->vid_win.height, 255);
     if (camera->greyscale){
       for (i = 0; i < camera->vid_win.width*camera->vid_win.height; i++){
@@ -364,7 +404,7 @@ void ppm_save(struct Camera *camera)
   }
 
   /*  Ascii  */
-  else if (camera->save_struct.format == ASCII){
+  else if (camera->save_struct.format == PPM_ASCII){
     fprintf (outfile, "P3\n%d %d\n%d\n", camera->vid_win.width, camera->vid_win.height, 255);
     if (camera->greyscale) {
       for (i = 0; i < camera->vid_win.width*camera->vid_win.height; i++){ 
@@ -435,45 +475,42 @@ void jpeg_save(struct Camera *camera)
   jpeg_finish_compress(&cinfo);
   jpeg_destroy_compress(&cinfo);
   
+  fclose(outfile);
+  return;
+}
 
-/*
+void raw_save(struct Camera *camera)
+{
+  FILE *outfile;
+  int ls=0, x, y;
+  int i;
+  int lines, pixels_per_line;
+  unsigned char buff[3];
   struct jpeg_compress_struct cinfo;
   struct jpeg_error_mgr jerr;
   JSAMPROW row_pointer[1];     // pointer to JSAMPLE row(s)
   int row_stride;              // physical row width in image buffer
 
-  cinfo.err = jpeg_std_error(&jerr);
-  jpeg_create_compress(&cinfo);
-  
-  jpeg_stdio_dest(&cinfo, outf);
-  cinfo.image_width = camera->vid_win.width;
-  cinfo.image_height = camera->vid_win.height;
+  if(!strcmp(camera->savefile, ""))
+    outfile = stdout;
+  else if(camera->savefile != NULL)
+    outfile = fopen(camera->savefile, "w");
 
-  if (camera->greyscale) {
-    cinfo.input_components = 1;   
-    cinfo.in_color_space = JCS_GRAYSCALE;     
-  }
-  else {
-    cinfo.input_components = 3;
-    cinfo.in_color_space = JCS_RGB;
+  if (outfile == NULL){
+    errordialog("Could not open file");
+    return;
   }
 
-  cinfo.optimize_coding = TRUE;
-  jpeg_set_defaults(&cinfo);
-  jpeg_set_quality(&cinfo, 75, TRUE );
-  jpeg_start_compress(&cinfo, TRUE);
-  row_stride = cinfo.input_components*camera->vid_win.width;
-  while (cinfo.next_scanline < cinfo.image_height) {
-    row_pointer[0] = &camera->pic[cinfo.next_scanline * row_stride];
-    jpeg_write_scanlines(&cinfo, row_pointer, 1);
-  }
-  jpeg_finish_compress(&cinfo);
-  jpeg_destroy_compress(&cinfo);
-*/
+  if (fwrite(camera->picbuff, sizeof(unsigned char), (size_t)camera->img_size, outfile) == camera->img_size)
+	fprintf(stderr, "raw_save %d bytes... success\n", (int)camera->img_size);
+  else
+  	fprintf(stderr, "raw_save error\n");
 
   fclose(outfile);
+  
   return;
 }
+
 void save_cancel(GtkWidget *widget, struct Camera *camera)
 {
   gtk_widget_destroy(widget->parent->parent->parent);
@@ -494,7 +531,7 @@ void switch_page_png(GtkWidget *widget, struct Camera *camera)
   gtk_widget_hide(camera->currentsavepage);
   gtk_widget_show(camera->page_png.frame_png);
   camera->currentsavepage = camera->page_png.frame_png;
-  camera->savetype = PNG;
+  camera->savetype = SAVE_PNG;
 }
 
 void switch_page_jpeg(GtkWidget *widget, struct Camera *camera)
@@ -502,15 +539,22 @@ void switch_page_jpeg(GtkWidget *widget, struct Camera *camera)
   gtk_widget_hide(camera->currentsavepage);
   gtk_widget_show(camera->page_jpeg.frame_jpeg);
   camera->currentsavepage = camera->page_jpeg.frame_jpeg;
-  camera->savetype = JPEG;
+  camera->savetype = SAVE_JPEG;
 }
 
+void switch_page_raw(GtkWidget *widget, struct Camera *camera)
+{
+  gtk_widget_hide(camera->currentsavepage);
+  camera->savetype = SAVE_RAW;
+}
+
+
 void switch_page_ppm(GtkWidget *widget, struct Camera *camera)
 {
   gtk_widget_hide(camera->currentsavepage);
   gtk_widget_show(camera->page_ppm.frame_ppm);
   camera->currentsavepage = camera->page_ppm.frame_ppm;
-  camera->savetype = PPM;
+  camera->savetype = SAVE_PPM;
 }
 
 void save_dialog(GtkWidget *widget, struct Camera *camera)
@@ -524,6 +568,7 @@ void save_dialog(GtkWidget *widget, struct Camera *camera)
   GtkWidget *typemenubutton;
   GtkWidget *ppmitem;
   GtkWidget *jpegitem;
+  GtkWidget *rawitem;
   GtkWidget *pngitem;
   //int openpage = camera->savetype;
 
@@ -584,6 +629,13 @@ void save_dialog(GtkWidget *widget, struct Camera *camera)
                      GTK_SIGNAL_FUNC (switch_page_jpeg), camera);
   gtk_widget_show(jpegitem);
 
+  rawitem = gtk_menu_item_new_with_label("raw");
+  gtk_menu_append(GTK_MENU(typemenu), rawitem);
+  gtk_signal_connect(GTK_OBJECT (rawitem), "activate",
+                    GTK_SIGNAL_FUNC (switch_page_raw), camera);
+  gtk_widget_show(rawitem);
+  
+  
   gtk_option_menu_set_menu(GTK_OPTION_MENU(typemenubutton), typemenu);
   gtk_option_menu_set_history(GTK_OPTION_MENU(typemenubutton), camera->savetype);
 
@@ -612,7 +664,7 @@ void save_dialog(GtkWidget *widget, struct Camera *camera)
   gtk_box_pack_start(GTK_BOX(camera->page_ppm.vbox_ppm), camera->page_ppm.radio_ascii_ppm, FALSE, FALSE, 2);
   gtk_widget_show(camera->page_ppm.radio_ascii_ppm);
 
-  if(camera->savetype == PPM){
+  if(camera->savetype == SAVE_PPM){
     gtk_widget_show(camera->page_ppm.frame_ppm);
     camera->currentsavepage = camera->page_ppm.frame_ppm;
   }
@@ -661,7 +713,7 @@ void save_dialog(GtkWidget *widget, struct Camera *camera)
   gtk_widget_show(camera->page_jpeg.optimize_jpeg);
   gtk_box_pack_start(GTK_BOX(vbox), camera->page_jpeg.frame_jpeg, FALSE, FALSE, 2);
 
-  if(camera->savetype == JPEG){
+  if(camera->savetype == SAVE_JPEG){
     gtk_widget_show(camera->page_jpeg.frame_jpeg);
     camera->currentsavepage = camera->page_jpeg.frame_jpeg;
   }
@@ -699,7 +751,7 @@ void save_dialog(GtkWidget *widget, struct Camera *camera)
   
   gtk_box_pack_start(GTK_BOX(vbox), camera->page_png.frame_png, FALSE, FALSE, 2);
 
-  if(camera->savetype == PNG){
+  if(camera->savetype == SAVE_PNG){
     gtk_widget_show(camera->page_png.frame_png);
     camera->currentsavepage = camera->page_png.frame_png;
   }
@@ -752,6 +804,10 @@ void save_dialog(GtkWidget *widget, struct Camera *camera)
   return;
 }
 
+/* 
+ * Append timestamp to filename (before the first occurence 
+ * of '.' if there is any) 
+ */
 void savefile_append_time(struct Camera *camera)
 {
   char *dotlocation;
@@ -764,10 +820,10 @@ void savefile_append_time(struct Camera *camera)
   else
     length = (dotlocation - camera->savefileclean)/(sizeof(char));
   strncpy(prefix, camera->savefileclean, length);
-  sprintf(curtime, ".%d", time(NULL));
+  sprintf(curtime, "-%d", time(NULL));
   strcpy(prefix+length, curtime);
   strcpy(prefix+length+strlen(curtime), camera->savefileclean+length);
-  //printf("%s\n", prefix);
+  //  printf("%s\n", prefix);
   
   strcpy(camera->savefile, prefix);
   
